@@ -1,4 +1,11 @@
-/* RFCNB Standard includes ... */
+/*
+ * Copyright (C) 1996-2017 The Squid Software Foundation and contributors
+ *
+ * Squid software is distributed under GPLv2+ license and includes
+ * contributions from numerous individuals and organizations.
+ * Please see the COPYING and CONTRIBUTORS files for details.
+ */
+
 /*
  *
  * RFCNB Standard Includes
@@ -6,8 +13,6 @@
  * Copyright (C) 1996, Richard Sharpe
  */
 
-/* One day we will conditionalize these on OS types ... */
-
 /*
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,9 +28,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
+
 #ifndef _RFCNB_STD_INCLUDES_H
 #define _RFCNB_STD_INCLUDES_H
 
+/* RFCNB Standard includes ... */
+/* One day we will conditionalize these on OS types ... */
+
 #define BOOL int
 typedef short int16;
 
@@ -41,7 +50,6 @@ typedef short int16;
 #endif
 #include <signal.h>
 #include <errno.h>
-#include <stdio.h>
 #include <unistd.h>
 
 #define TRUE 1
@@ -54,3 +62,4 @@ typedef short int16;
 #endif
 
 #endif /* _RFCNB_STD_INCLUDES_H */
+
